using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using WMPLib;
using System.Runtime.InteropServices;

namespace WindowsApplication4
{
    public partial class MainFrm : Form
    {
        [DllImport("user32", CharSet = CharSet.Auto)]
        private static extern bool SystemParametersInfo(int uAction, int uParam, ref bool lpvParam, int fuWinIni);


        
        const int SPI_SETSCREENSAVEACTIVE = 17;
        const int SPI_GETSCREENSAVEACTIVE = 16;

        bool screenSaverActive;
        
        
        const string buttonNamePrefix = "listBtn";
        const string newFileDescription = "Neue Zusammenstellung";
        const string effectIdentifier = "-effect";

        string currentFile;
        
        string currentListKey;

        enum FadeOutAction
        {
            Next,
            Stop
        }

        struct PlayListEntry
        {
            public string baseName;
            public string fullPath;
            public int cntPlayed;
        }

        System.Collections.SortedList playLists;
        System.Collections.SortedList descriptionList;
        System.Collections.SortedList effectList;

        public MainFrm()
        {
            InitializeComponent();
        }


        public static List<T> randomizeList<T>(List<T> list)
        {
            List<T> randomizedList = new List<T>();
            Random rnd = new Random();

            if (list.Count > 1)
            {
                while (list.Count > 0)
                {
                    int index = rnd.Next(0, list.Count); //pick a random item from the master list
                    randomizedList.Add(list[index]); //place it at the end of the randomized list
                    list.RemoveAt(index);
                }
            }
            else
            {
                randomizedList = list;
            }
            return randomizedList;
        }

        private void playEffectOverlay(List<PlayListEntry> list)
        {
            WindowsMediaPlayerClass effectPlayer = new WindowsMediaPlayerClass();
            ;
            if (list.Count > 0)
            {
                list = randomizeList<PlayListEntry>(list);

                effectPlayer.settings.volume = (int)UpDwnEffectVolume.Value;
                effectPlayer.URL = (list[0].fullPath);
                effectPlayer.play();
            }

/*            while (effectPlayer.playState != WMPPlayState.wmppsStopped)
            {
                Application.DoEvents();
            }*/
        }

        private void fadeOut(FadeOutAction action)
        {

            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            
            const int fadeOutSec = 2;
            const double fadeTimeStepSec = 0.1;


            if (WindowsMediaPlayer.currentMedia != null)
            {

                int oldVolume = WindowsMediaPlayer.settings.volume;

                double lenghtSec = WindowsMediaPlayer.currentMedia.duration;
                double curPosSec = WindowsMediaPlayer.Ctlcontrols.currentPosition;

                double turndDownSteps = 0;

                if (lenghtSec - curPosSec < fadeOutSec)
                {
                    turndDownSteps = lenghtSec - curPosSec;
                }
                else
                {
                    turndDownSteps = fadeOutSec;
                }

                if (turndDownSteps > 0)
                {
                    double curStep = 0;
                    int volumeReduce = 0;
                    for (curStep = turndDownSteps; curStep > 0; curStep -= fadeTimeStepSec)
                    {
                        volumeReduce = (int)(WindowsMediaPlayer.settings.volume / curStep * fadeTimeStepSec);
                        WindowsMediaPlayer.settings.volume = WindowsMediaPlayer.settings.volume - volumeReduce;
                        System.Threading.Thread.Sleep((int)(1000 * fadeTimeStepSec));
                    }
                }

                switch (action = FadeOutAction.Next)
                {
                    case FadeOutAction.Next:
                        WindowsMediaPlayer.Ctlcontrols.next();
                        break;

                    case FadeOutAction.Stop:
                        WindowsMediaPlayer.Ctlcontrols.stop();
                        break;

                }

                WindowsMediaPlayer.settings.volume = oldVolume;
            }

            this.Cursor = System.Windows.Forms.Cursors.Default;

            
        }

        private void songListClicked(object sender, EventArgs e)
        {
            ToolStripButton tsb;
            string          oldKey = currentListKey;
            foreach (ToolStripItem btn in PlayListToolBar.Items)
            {
                if (btn.Equals(sender))
                {
                    ((ToolStripButton)btn).CheckState = CheckState.Checked;
                }
                else if (btn.GetType() == typeof(ToolStripButton))
                {
                    tsb = (ToolStripButton)btn;
                    tsb.CheckState = System.Windows.Forms.CheckState.Unchecked;
                }

            }
            currentListKey = ((ToolStripButton)sender).Text;

            int i = 0;
            if (oldKey != currentListKey)
            {

                FileListView.Items.Clear();

                ListViewItem listViewItem;

                for (i = 0; i < ((List<PlayListEntry>)playLists[currentListKey]).Count; i++)
                {
                    listViewItem = FileListView.Items.Add(((List<PlayListEntry>)playLists[currentListKey])[i].fullPath);
                    listViewItem.SubItems.Add(((List<PlayListEntry>)playLists[currentListKey])[i].baseName);
                    listViewItem.SubItems.Add(currentListKey);
                }
                TXTListDescription.Text = (string)descriptionList[currentListKey];
                ChckBxEffect.Checked = (bool)effectList[currentListKey];
                LblListDescription.Text = "Beschreibung fr Liste " + currentListKey;
            }

            if (ProgramMode.SelectedIndex == 1)
            {
                if (oldKey == currentListKey &&
                    WindowsMediaPlayer.currentPlaylist.count > 0)
                {
                    fadeOut(FadeOutAction.Next);
                }
                else
                {
                    fadeOut(FadeOutAction.Stop);
                    
                    WindowsMediaPlayer.currentPlaylist.clear();

                    playLists[currentListKey] = randomizeList<PlayListEntry>((List<PlayListEntry>)playLists[currentListKey]);

                    for (i =0; i < ((List<PlayListEntry>)playLists[currentListKey]).Count; i++)
                    {
                        WindowsMediaPlayer.currentPlaylist.appendItem(WindowsMediaPlayer.newMedia(((List<PlayListEntry>)playLists[currentListKey])[i].fullPath));
                    }
                    if (WindowsMediaPlayer.currentPlaylist.count >= 1)
                    {
                        WindowsMediaPlayer.Ctlcontrols.play();
                    }
                }
            }
        }

        void clearPlayLists()
        {
            playLists = new System.Collections.SortedList();
            descriptionList = new System.Collections.SortedList();
            effectList = new System.Collections.SortedList();

            int i = 0;
            for (i = 0; i <= 9; i++)
            {
                playLists.Add(i.ToString(), new List<PlayListEntry>());
                descriptionList.Add(i.ToString(), "");
                effectList.Add(i.ToString(), false);
            }

            for (i = char.ConvertToUtf32("a", 0); i <= char.ConvertToUtf32("z", 0); i++)
            {
                playLists.Add(char.ConvertFromUtf32(i), new List<PlayListEntry>());
                descriptionList.Add(char.ConvertFromUtf32(i), "");
                effectList.Add(char.ConvertFromUtf32(i), false);
            }
            setAccessKeyToolTip();
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            currentFile = "";

            SaveFileDialog.Filter = Application.ProductName + " (*.hqm)|*.hqm|Alle Dateien (*.*)|*.*";
            OpenFileDialog.Filter = SaveFileDialog.Filter;

            LblVersion.Text = "V. " + Application.ProductVersion;

            ToolStripButton btn;

            UpDwnMusicVolume.Value = WindowsMediaPlayer.settings.volume;
            UpDwnEffectVolume.Value = UpDwnMusicVolume.Value + (UpDwnEffectVolume.Maximum - UpDwnMusicVolume.Value) / 2;

            int i = 0;
            for (i = 0; i <= 9; i++)
            {
                btn = new ToolStripButton(i.ToString());
                btn.CheckOnClick = true;
                btn.Name = buttonNamePrefix + btn.Text;
                btn.Click += new EventHandler(songListClicked);
                PlayListToolBar.Items.Add(btn);
            }
            for (i = char.ConvertToUtf32("a" , 0); i <= char.ConvertToUtf32("z", 0); i++)
            {
                btn = new ToolStripButton(char.ConvertFromUtf32(i));
                btn.CheckOnClick = true;
                btn.Name = buttonNamePrefix + btn.Text;
                btn.Click += new EventHandler(songListClicked);
                PlayListToolBar.Items.Add(btn);

            }
            clearPlayLists();


            ProgramMode.SelectedIndex = 0;

            btn = (ToolStripButton)PlayListToolBar.Items.Find(buttonNamePrefix + "0", false)[0];
            songListClicked(btn, null);
            btn.CheckState = CheckState.Checked;
            this.Text = Application.ProductName + " - " + newFileDescription;

        }

        private void FileListView_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop) &&
                ProgramMode.SelectedIndex == 0)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void FileListView_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop) && ProgramMode.SelectedIndex == 0)
            {
                PlayListEntry entry;
                FileInfo fi;
                ListViewItem listViewItem;
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                foreach (string file in files)
                {
                    fi = new FileInfo(file);
                    listViewItem = FileListView.Items.Add(fi.FullName);
                    listViewItem.SubItems.Add(fi.Name);
                    listViewItem.SubItems.Add(currentListKey);

                    entry = new PlayListEntry();
                    entry.fullPath = fi.FullName;
                    entry.baseName = fi.Name;
                    entry.cntPlayed = 0;
                    
                    ((List<PlayListEntry>)playLists[currentListKey]).Add(entry);
                }

            }
        }

        private void FileListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (ProgramMode.SelectedIndex == 0)
            {

                if (e.KeyCode == Keys.Delete && ((ListView)sender).SelectedItems.Count > 0)
                {
                    string removedItem;
                    removedItem = ((ListView)sender).SelectedItems[0].Text;
                    ((ListView)sender).SelectedItems[0].Remove();
                    int i = 0;
                    for (i = 0; i < ((List<PlayListEntry>)playLists[currentListKey]).Count; i++)
                    {
                        if (((List<PlayListEntry>)playLists[currentListKey])[i].fullPath == removedItem)
                        {
                            ((List<PlayListEntry>)playLists[currentListKey]).Remove(((List<PlayListEntry>)playLists[currentListKey])[i]);
                            break; // only one item
                        }

                    }
                }
            }
        }

        private void MainFrm_KeyDown(object sender, KeyEventArgs e)
        {
            if (ProgramMode.SelectedIndex == 1)
            {
                NumericUpDown   upDownCtrl;

                int smothModeDiv = 1;

                if (e.Shift)
                {
                    smothModeDiv = 10;
                }

                if (e.Alt != true)
                {
                    upDownCtrl = UpDwnMusicVolume;
                }
                else
                {
                    upDownCtrl = UpDwnEffectVolume;
                }
                
                
                if (e.KeyCode == Keys.PageDown)
                {
                    if (upDownCtrl.Value < upDownCtrl.Minimum + (10 / smothModeDiv))
                        upDownCtrl.Value = upDownCtrl.Minimum;
                    else
                        upDownCtrl.Value -= (10 / smothModeDiv);

                }
                else if (e.KeyCode == Keys.PageUp)
                {
                    if (upDownCtrl.Value > upDownCtrl.Maximum - (10 / smothModeDiv))
                        upDownCtrl.Value = upDownCtrl.Maximum;
                    else
                        upDownCtrl.Value += (10 / smothModeDiv);

                }
                

                if (e.KeyCode == Keys.Space)
                {
                    if (WindowsMediaPlayer.playState == WMPPlayState.wmppsPlaying)
                    {
                        WindowsMediaPlayer.Ctlcontrols.pause();
                    }
                    else if (WindowsMediaPlayer.playState == WMPPlayState.wmppsPaused)
                    {
                        WindowsMediaPlayer.Ctlcontrols.play();
                    }
                }
                else
                {
                    string key = e.KeyCode.ToString().ToLower().Replace("numpad", "");
                    if (key.Length > 1)
                    {
                        key = key.Replace("d", "");
                    }

                    
                    foreach (ToolStripItem btn in PlayListToolBar.Items)
                    {
                        if (btn.GetType() == typeof(ToolStripButton))
                        {
                            if (((ToolStripButton)btn).Text == key)
                            {
                                if (e.Alt == true || (bool)effectList[((ToolStripButton)btn).Text] == true)
                                {
                                    List<PlayListEntry> newList = new List<PlayListEntry>();
                                    int i;
                                    for (i = 0; i < ((List<PlayListEntry>)playLists[((ToolStripButton)btn).Text]).Count; i++)
                                    {
                                        newList.Add(((List<PlayListEntry>)playLists[((ToolStripButton)btn).Text])[i]);
                                    }
                                    playEffectOverlay(newList);
                                }
                                else
                                {
                                    songListClicked((ToolStripButton)btn, null);
                                }                                
                                break;
                            }
                        }
                    }
                }
            }
        }

        private void ProgramMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TXTListDescription.ReadOnly = ((ToolStripComboBox)sender).SelectedIndex == 1;

            BtnOpen.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;
            BtnSave.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;
            BtnSaveAs.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;
            BtnNew.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;

            UpDwnMusicVolume.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;
            UpDwnEffectVolume.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;
            ChckBxEffect.Enabled = ((ToolStripComboBox)sender).SelectedIndex == 0;


            bool dummy = false;
            if (((ToolStripComboBox)sender).SelectedIndex == 0)
            {
                checkEnableScreenSaver();


                if (WindowsMediaPlayer.playState != WMPPlayState.wmppsStopped)
                {
                    WindowsMediaPlayer.Ctlcontrols.stop();
                }
            }
            else
            {

                
                SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0, ref screenSaverActive, 0);
                if (screenSaverActive)
                {
                    SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 1, ref dummy, 0);
                }
            }
            TXTListDescription.Focus();
            TXTHlp.Focus();



        }

        void checkEnableScreenSaver()
        {
            bool dummy = false;
            
            if (screenSaverActive)
            {
                SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 1, ref dummy, 0);
            }
        }


        private bool saveFile(string fileName)
        {
            StreamWriter myFile = new StreamWriter(fileName);
            int i;
            int j;
            string titleStr;

            bool ret = false;

            try
            {
                for (i = 0; i <= 9; i++)
                {
                    titleStr = "[" + i.ToString();
                    titleStr += (bool)effectList[i.ToString()] ? effectIdentifier : "";
                    titleStr += "]";
                    titleStr += (string)descriptionList[i.ToString()];


                    myFile.WriteLine(titleStr);
                    for (j = 0; j < ((List<PlayListEntry>)playLists[i.ToString()]).Count; j++)
                    {
                        myFile.WriteLine(((List<PlayListEntry>)playLists[i.ToString()])[j].fullPath);
                    }
                }

                for (i = char.ConvertToUtf32("a", 0); i <= char.ConvertToUtf32("z", 0); i++)
                {
                    titleStr = "[" + char.ConvertFromUtf32(i);
                    titleStr += (bool)effectList[char.ConvertFromUtf32(i)] ? effectIdentifier : "";
                    titleStr += "]";
                    titleStr += (string)descriptionList[char.ConvertFromUtf32(i)];
                    
                    myFile.WriteLine(titleStr);
                    for (j = 0; j < ((List<PlayListEntry>)playLists[char.ConvertFromUtf32(i)]).Count; j++)
                    {
                        myFile.WriteLine(((List<PlayListEntry>)playLists[char.ConvertFromUtf32(i)])[j].fullPath);
                    }
                }
                ret = true;

            }
            catch (Exception ex)
            {
                MessageBox.Show("Datei kann nicht gespeichert werden: " + fileName + "\n" + ex.Message, "Speichern", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            finally
            {
                if (myFile != null)
                {
                    myFile.Close();
                }
            }
            return ret;
        }

        private void setAccessKeyToolTip()
        {
            ToolStripButton tbb;
            ToolStripItem tsi;
            int i = 0;
            for (i = 0; i < PlayListToolBar.Items.Count; i++)
            {
                tsi = PlayListToolBar.Items[i];
                tbb = (ToolStripButton)tsi;
                tbb.ToolTipText = (string)descriptionList[tbb.Text];
            }
        }

        private bool loadFile(string fileName)
        {
            StreamReader myFile = new StreamReader(fileName, System.Text.Encoding.UTF8);

            string curKey = "";
            
            bool ret = false ;
            string curLine;

            try
            {
                clearPlayLists();

                PlayListEntry entry;

                while (!myFile.EndOfStream)
                {
                    curLine = myFile.ReadLine().ToLower();
                    if (curLine.StartsWith("["))
                    {
                        curKey = curLine.Substring(1, 1);
                        effectList[curKey] = curLine.Contains(effectIdentifier +"]");

                        curLine = curLine.Replace(effectIdentifier + "]", "]");

                        descriptionList[curKey] = curLine.Substring(3, curLine.Length - 3);
                    }
                    else if (curLine != "")
                    {
                        FileInfo fileInfo = new FileInfo(curLine);

                        entry = new PlayListEntry();
                        entry.fullPath = curLine;
                        
                        if (fileInfo != null && System.IO.File.Exists(curLine))
                        {
                            entry.baseName = fileInfo.Name;
                        }
                        else
                        {
                            entry.baseName = "Datei nicht gefunden";
                        }
                        entry.cntPlayed = 0;
                        
                        ((List<PlayListEntry>)playLists[curKey]).Add(entry);
                    }
                }
                ret = true;

            }
            catch (Exception ex)
            {
                MessageBox.Show("Datei kann nicht geffnet werden: " + fileName + "\n" + ex.Message, "ffnen", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                BtnNew_Click(BtnNew, null);
            }
            finally
            {
                if (myFile != null)
                {
                    myFile.Close();
                }
                setAccessKeyToolTip();
            }
            return ret;
        }

        
        private void BtnNew_Click(object sender, EventArgs e)
        {
            currentFile = "";

            
            FileListView.Items.Clear();
            clearPlayLists();

            ToolStripButton btn;

            currentListKey = "1";
            btn = (ToolStripButton)PlayListToolBar.Items.Find(buttonNamePrefix + "0", false)[0];
            songListClicked(btn, null);

            this.Text = Application.ProductName + " - " + newFileDescription;
        }

        private void BtnSaveAs_Click(object sender, EventArgs e)
        {
            if (SaveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (saveFile(SaveFileDialog.FileName))
                {
                    FileInfo fi = new FileInfo(SaveFileDialog.FileName);

                    this.Text = Application.ProductName + " - " + fi.Name;
                    currentFile = fi.FullName;
                }
            }
        }

        private void BtnOpen_Click(object sender, EventArgs e)
        {
            if (OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (loadFile(OpenFileDialog.FileName))
                {
                    FileInfo fi = new FileInfo(OpenFileDialog.FileName);

                    ToolStripButton btn;

                    currentListKey = "1";
                    btn = (ToolStripButton)PlayListToolBar.Items.Find(buttonNamePrefix + "0", false)[0];
                    songListClicked(btn, null);
                    this.Text = Application.ProductName + " - " + fi.Name;
                    currentFile = fi.FullName;
                }
            }

        }

        private void BtnSave_Click(object sender, EventArgs e)
        {
            if (currentFile == "")
            {
                BtnSaveAs_Click(BtnSaveAs, e);
            }
            else 
            {
                saveFile(currentFile);
            }
        }

        private void TXTListDescription_TextChanged(object sender, EventArgs e)
        {
            if (currentListKey != null)
            {
                descriptionList[currentListKey] = ((TextBox)sender).Text;
                ((ToolStripButton)PlayListToolBar.Items.Find(buttonNamePrefix + currentListKey, false)[0]).ToolTipText = ((TextBox)sender).Text;
            }
        }

        private void MainFrm_FormClosing(object sender, FormClosingEventArgs e)
        {
            checkEnableScreenSaver();
        }

        private void TrkBarVolume_ValueChanged(object sender, EventArgs e)
        {
            WindowsMediaPlayer.settings.volume = ((TrackBar)sender).Value;
        }

        private void BTNExplorer_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("explorer");
        }

        private void ChckBxEffect_CheckedChanged(object sender, EventArgs e)
        {
            if (currentListKey != null)
            {
                effectList[currentListKey] = ((CheckBox)sender).Checked;
            }
        }


    }
}